--[==[

[Tetris]
code = for Love2D 0.10.0
licence = zlib
email = ewuzg at 163.com

--]==]

----------------------------------------------------------------------------------
-- version:
local Tetris = {
Version = "1.00",
Update  = "15:47 2016/5/18"
}
local Tetris_Soft_info = "[UPDATE: " .. Tetris.Update .. "]"
function show_soft_info(w, h)
	love.graphics.setColor(128, 128, 128)
	love.graphics.print(Tetris_Soft_info, w - 280, h - 22)
end
----------------------------------------------------------------------------------

-- os:
scr_width, scr_height, scr_flags = love.window.getMode()
os_str = love.system.getOS()

-- json:
local json = require ("dkjson")
local use_json = true
local file_json_name = "tetris.json"
local json_tbl = {time = "0-0-0", timeused = 0, timeusedstr = "0", line = 0, score = 0, sound = false, aux = false}

-- tetris dimension:
block_size    = 24
block_x_count = 12
block_y_count = 20
frame_width  = block_x_count * block_size
frame_height = block_y_count * block_size

frame_next_width  = block_size * 4
frame_next_height = frame_next_width
frame_indent_x    = 10

frame_beg_x = (scr_width - frame_width) / 2
frame_beg_y = (scr_height - frame_height) / 2
frame_end_x = frame_beg_x + frame_width
frame_end_y = frame_beg_y + frame_height

frame_next_beg_x = frame_end_x + frame_indent_x
frame_next_beg_y = frame_beg_y
block_next_x_count = 4
block_next_y_count = 4
frame_next_end_x = frame_next_beg_x + block_next_x_count * block_size
frame_next_end_y = frame_next_beg_y + block_next_y_count * block_size

-- tetris color & font:
tetris_line_color = 128
tetris_font_color = 192
tetris_font       = love.graphics.newFont("font.ttf", 18);

local tetris_color_tbl = {
{255, 0, 0},
{0, 255, 0},
{0, 0, 255},
{255, 255, 0},
{255, 0, 255},
{0, 255, 255},
{255, 255, 255},
}

-- tetris state:
----------------------------
enum_tetris_state_live  = 1
enum_tetris_state_pause = 2
enum_tetris_state_die   = 3
----------------------------
tetris_state = enum_tetris_state_live

tetris_block_next_type = 1
tetris_block_next_type_index = 1
tetris_block_next_x = 0
tetris_block_next_y = 0

tetris_block_cur_type = 1
tetris_block_cur_type_index = 1
tetris_block_cur_ix = 0
tetris_block_cur_iy = 0

tetris_line  = 0
tetris_score = 0
game_start_time = 0
game_end_time   = 0
game_duration_str = "0:00:00"
game_pause_start_time = 0
game_pause_duration = 0

---------------------------------
-- dlg_confirm_quit:

game_pause_confirm_quit = false
dlg_confirm_quit_pos = {
	w = frame_width, h = 160,
	x = frame_beg_x, y = (scr_height - 200) / 2,
}
dlg_confirm_quit_btn_w = 100
dlg_confirm_quit_btn_h = 30
dlg_confirm_quit_btnok_pos = {
	x = (dlg_confirm_quit_pos.x + 20),
	y = (dlg_confirm_quit_pos.y + (dlg_confirm_quit_pos.h - dlg_confirm_quit_btn_h) / 2 + 32),
}
dlg_confirm_quit_btncancel_pos = {
	x = (dlg_confirm_quit_pos.x + 20 + dlg_confirm_quit_btn_w + 48),
	y = dlg_confirm_quit_btnok_pos.y,
}

function draw_dlg_confirm_quit()
	love.graphics.setColor(tetris_line_color, tetris_line_color, tetris_line_color)
	love.graphics.rectangle("fill", dlg_confirm_quit_pos.x, dlg_confirm_quit_pos.y, dlg_confirm_quit_pos.w, dlg_confirm_quit_pos.h)
	love.graphics.setColor(tetris_line_color-32, tetris_line_color-32, tetris_line_color-32)
	love.graphics.rectangle("fill", dlg_confirm_quit_btnok_pos.x, dlg_confirm_quit_btnok_pos.y, dlg_confirm_quit_btn_w, dlg_confirm_quit_btn_h)
	love.graphics.rectangle("fill", dlg_confirm_quit_btncancel_pos.x, dlg_confirm_quit_btncancel_pos.y, dlg_confirm_quit_btn_w, dlg_confirm_quit_btn_h)
	love.graphics.setColor(tetris_font_color, tetris_font_color, tetris_font_color)
	love.graphics.print("ARE YOU SURE TO QUIT?", (dlg_confirm_quit_pos.x + 16), (dlg_confirm_quit_pos.y + 32))
	love.graphics.print("OK", (dlg_confirm_quit_btnok_pos.x + 36), (dlg_confirm_quit_btnok_pos.y + 2))
	love.graphics.print("CANCEL", (dlg_confirm_quit_btncancel_pos.x + 10), (dlg_confirm_quit_btncancel_pos.y + 2))
end

enum_dlg_confirm_btn_ok     = 1
enum_dlg_confirm_btn_cancel = 2
enum_dlg_confirm_btn_none   = 3

function dlg_confirm_quit_click(x, y)
	local nRet = enum_dlg_confirm_btn_none
	if (x >= dlg_confirm_quit_btnok_pos.x) and (x <= (dlg_confirm_quit_btnok_pos.x + dlg_confirm_quit_btn_w)) and
		(y >= dlg_confirm_quit_btnok_pos.y) and (y <= (dlg_confirm_quit_btnok_pos.y + dlg_confirm_quit_btn_h)) then
		nRet = enum_dlg_confirm_btn_ok
	elseif (x >= dlg_confirm_quit_btncancel_pos.x) and (x <= (dlg_confirm_quit_btncancel_pos.x + dlg_confirm_quit_btn_w)) and
			(y >= dlg_confirm_quit_btncancel_pos.y) and (y <= (dlg_confirm_quit_btncancel_pos.y + dlg_confirm_quit_btn_h)) then
		nRet = enum_dlg_confirm_btn_cancel
	end
	return nRet
end
---------------------------------
-- aux:
local tetris_aux = false
local tetris_aux_line_beg_x = 0
local tetris_aux_line_end_x = 0
---------------------------------


--[==[

1. #@##

2. #
   @##

3.   #
   ##@

4. @#
   ##

5.  @#
   ##

6.  #
   #@#

7. #@
    ##

--]==]


local tetris_block_tbl = {
{block = 4,
 coordination = {
 {{x = -1, y = 0},{x = 1, y = 0},{x = 2, y = 0}},
 {{x = 0, y = -1},{x = 0, y = 1},{x = 0, y = 2}},
 {{x = -1, y = 0},{x = 1, y = 0},{x = 2, y = 0}},
 {{x = 0, y = -1},{x = 0, y = 1},{x = 0, y = 2}}
 }, color = 1},

{block = 4,
 coordination = {
 {{x = 0, y = -1},{x =  1, y = 0},{x = 2, y =  0}},
 {{x =  1, y = 0},{x =  0, y = 1},{x = 0, y =  2}},
 {{x = -2, y = 0},{x = -1, y = 0},{x = 0, y =  1}},
 {{x = -1, y = 0},{x = 0, y = -1},{x = 0, y = -2}}
 }, color = 2},

{block = 4,
 coordination = {
 {{x = -2,y =  0},{x = -1, y = 0},{x = 0, y = -1}},
 {{x = 0, y = -2},{x = 0, y = -1},{x = 1, y =  0}},
 {{x = 1, y =  0},{x = 2, y =  0},{x = 0, y =  1}},
 {{x = -1,y =  0},{x = 0, y =  1},{x = 0, y =  2}}
 }, color = 3},

{block = 4,
 coordination = {
 {{x = 1, y = 0}, {x = 0, y = 1},{x = 1, y = 1}},
 {{x = 1, y = 0}, {x = 0, y = 1},{x = 1, y = 1}},
 {{x = 1, y = 0}, {x = 0, y = 1},{x = 1, y = 1}},
 {{x = 1, y = 0}, {x = 0, y = 1},{x = 1, y = 1}}
 }, color = 4},

{block = 4,
 coordination = {
 {{x =  1, y =  0},{x =  0, y =  1},{x = -1, y =  1}},
 {{x = -1, y = -1},{x = -1, y =  0},{x =  0, y =  1}},
 {{x = -1, y =  0},{x =  0, y = -1},{x =  1, y = -1}},
 {{x = 0,  y = -1},{x =  1, y =  0},{x =  1, y =  1}}
 }, color = 5},

{block = 4,
 coordination = {
 {{x = -1, y =  0},{x = 0, y = -1},{x = 1, y = 0}},
 {{x =  0, y = -1},{x = 1, y =  0},{x = 0, y = 1}},
 {{x = -1, y =  0},{x = 1, y =  0},{x = 0, y = 1}},
 {{x = -1, y =  0},{x = 0, y = -1},{x = 0, y = 1}}
 }, color = 6},

{block = 4,
 coordination = {
 {{x = -1, y =  0},{x =  0, y =  1},{x =  1, y = 1}},
 {{x =  0, y = -1},{x = -1, y =  0},{x = -1, y = 1}},
 {{x = -1, y = -1},{x =  0, y = -1},{x =  1, y = 0}},
 {{x =  1, y = -1},{x =  1, y =  0},{x =  0, y = 1}}
 }, color = 7}
}

local tetris_block_tbl_init  = {}
local tetris_block_frame_tbl = {}

-- ctrl area:
dir_bar_size = 30
dir_bar_x = frame_end_x + frame_indent_x
if os_str == "Android" then
	dir_bar_size = 80
	dir_bar_x = dir_bar_x + 15
	file_json_name = "/storage/emulated/0/tetris.json"
end
dir_bar_y = frame_end_y - dir_bar_size * 3

dir_bars = {
	{x = dir_bar_x + dir_bar_size, y = dir_bar_y, dir = "up"},
	{x = dir_bar_x + dir_bar_size * 2, y = dir_bar_y + dir_bar_size, dir = "right"},
	{x = dir_bar_x + dir_bar_size, y = dir_bar_y + dir_bar_size * 2, dir = "down"},
	{x = dir_bar_x, y = dir_bar_y + dir_bar_size, dir = "left"}
}

-- replay:
replay_bar_size = {w = dir_bar_size * 2, h = dir_bar_size}
replay_bar_pos  = {x = dir_bar_x, y = dir_bar_y - replay_bar_size.h - 20}

-- pause:
pause_bar_size = {w = dir_bar_size, h = dir_bar_size}
pause_bar_pos  = {x = (frame_beg_x - pause_bar_size.w - frame_indent_x - 15), y = (frame_end_y - pause_bar_size.h)}


-- info area:
info_rect_w = 180
info_rect_h = 200
info_rect = { x = frame_beg_x - info_rect_w - frame_indent_x, y = frame_beg_y,
			  w = info_rect_w, h = info_rect_h}
if os_str == "Android" then
	info_rect.x = info_rect.x - 15
end

-- diminish canvas:
diminish_canvas = love.graphics.newCanvas(frame_width, frame_height);
full_line_count = 0
full_line_tbl   = {}
diminish_tick   = 0

----------------------------------------------------------
-- fmt_duration:
function fmt_duration(duration)

	local d_sec = duration % 60;
	duration = math.floor(duration / 60);
	local d_min = duration % 60;
	duration = math.floor(duration / 60);
	local d_hour = duration;
	local str = string.format("T: %d:%.2d:%.2d", d_hour, d_min, d_sec);
	return str;

end

-- read_json:
function read_json()

	if use_json then

		-- json:
		local json_file = io.open(file_json_name)
		if json_file ~= nil then
			local str = json_file:read("*a")
			json_file:close()
			local obj, pos, err = json.decode (str, 1, nil)
			if not err then
				json_tbl.time = obj.time or "0-0-0"
				json_tbl.timeused = obj.timeused or 0
				json_tbl.timeusedstr = fmt_duration(json_tbl.timeused)
				json_tbl.line = obj.line or 0
				json_tbl.score = obj.score or 0
				json_tbl.sound = obj.sound or false
				json_tbl.aux = obj.aux or false
				tetris_aux = json_tbl.aux
			end
		end

	end

end

-- write_json:
function write_json(bIsGameOver)

	local bMaxUpdated = false
	if bIsGameOver then
		if tetris_line > json_tbl.line then
			json_tbl.line = tetris_line
			json_tbl.score = tetris_score
			bMaxUpdated = true
		end
	end

	if use_json then

		local json_file = io.open(file_json_name, "w")
		if json_file ~= nil then
			if bIsGameOver and bMaxUpdated then
				local dt = os.date("*t")
				local str = string.format("%d-%d-%d %d:%d:%d", dt.year, dt.month, dt.day,
										dt.hour, dt.min, dt.sec)
				json_tbl.time = str
				json_tbl.timeused = (os.time() - game_start_time - game_pause_duration)
				json_tbl.timeusedstr = fmt_duration(json_tbl.timeused)
				game_start_time = 0
			end
			local str_json = json.encode(json_tbl, {indent = true})
			json_file:write(str_json)
			json_file:close()
		end

	end

end
----------------------------------------------------------

-- init_tetris_block_tbl_init:
function init_tetris_block_tbl_init()

	for i=1,#tetris_block_tbl do
		tetris_block_tbl_init[i] = {}

		for j=1, #tetris_block_tbl[i].coordination do
			tetris_block_tbl_init[i][j] = {}
			local ix = 0
			local iy = 0

			for k=1, tetris_block_tbl[i].block - 1 do
				if ix > tetris_block_tbl[i].coordination[j][k].x then
					ix = tetris_block_tbl[i].coordination[j][k].x
				end
				if iy > tetris_block_tbl[i].coordination[j][k].y then
					iy = tetris_block_tbl[i].coordination[j][k].y
				end
			end

			tetris_block_tbl_init[i][j] = {x = math.abs(ix), y = math.abs(iy)}
		end

	end

end

-- init_tetris_block_frame_tbl:
function init_tetris_block_frame_tbl()

	for i=0, block_y_count-1 do
		tetris_block_frame_tbl[i] = {}
		for j=0, block_x_count-1 do
			tetris_block_frame_tbl[i][j] = 0
		end
	end

end

-- reload_tetris:
function reload_tetris()

	tetris_block_cur_type, tetris_block_cur_type_index = gen_rand_block()
	tetris_block_cur_ix, tetris_block_cur_iy = adjust_init_pos(tetris_block_cur_type, tetris_block_cur_type_index)
	tetris_block_cur_ix = tetris_block_cur_ix + block_x_count / 2 - 2

	gen_next_block()

	-- clear frame data:
	for i=0, block_y_count-1 do
		for j=0, block_x_count-1 do
			tetris_block_frame_tbl[i][j] = 0
		end
	end

	-- aux:
	if tetris_aux then
		tetris_aux_func_calc()
	end

	tetris_line  = 0
	tetris_score = 0
	game_start_time = os.time()
	game_end_time   = game_start_time
	game_pause_duration = 0
	tetris_state = enum_tetris_state_live

end

-- adjust_init_pos:
function adjust_init_pos(t, i)

	local ix = 0
	local iy = 0

	ix = tetris_block_tbl_init[t][i].x
	iy = tetris_block_tbl_init[t][i].y

	return ix, iy

end

-- draw_frame:
function draw_frame()

	love.graphics.setColor(tetris_line_color, tetris_line_color, tetris_line_color)
	local x = frame_beg_x
	for i = 1, block_x_count + 1 do
		love.graphics.line(x, frame_beg_y, x, frame_end_y)
		x = x + block_size
	end
	local y = frame_beg_y
	for i = 1, block_y_count + 1 do
		love.graphics.line(frame_beg_x, y, frame_end_x, y)
		y = y + block_size
	end

	x = frame_next_beg_x
	for i = 1, block_next_x_count + 1 do
		love.graphics.line(x, frame_next_beg_y, x, frame_next_end_y)
		x = x + block_size
	end
	y = frame_next_beg_y
	for i = 1, block_next_y_count + 1 do
		love.graphics.line(frame_next_beg_x, y, frame_next_end_x, y)
		y = y + block_size
	end

end

-- draw_dirbar:
function draw_dirbar()

	love.graphics.setColor(tetris_line_color, tetris_line_color, tetris_line_color)
	for i = 1, #dir_bars do
		love.graphics.rectangle("fill", dir_bars[i].x, dir_bars[i].y, dir_bar_size, dir_bar_size)
	end

	if os_str == "Android" then

		local quad_dd = (dir_bar_size / 4);

		-- left:
		local init_x = dir_bars[4].x;
		local init_y = dir_bars[4].y;
		local x_c = (init_x + quad_dd);
		local y_c = (init_y + quad_dd * 2);
		local x1  = (init_x + quad_dd * 2);
		local y1  = (init_y + quad_dd);
		local x2  = (init_x + quad_dd * 3);
		local y2  = (init_y + quad_dd * 2);
		local x3  = (init_x + quad_dd * 2);
		local y3  = (init_y + quad_dd * 3);
		love.graphics.setColor(tetris_font_color, tetris_font_color, tetris_font_color);
		love.graphics.line(x_c, y_c, x1, y1);
		love.graphics.line(x_c, y_c, x2, y2);
		love.graphics.line(x_c, y_c, x3, y3);

		-- right:
		init_x = dir_bars[2].x;
		init_y = dir_bars[2].y;
		x_c = (init_x + quad_dd * 3);
		y_c = (init_y + quad_dd * 2);
		x1  = (init_x + quad_dd * 2);
		y1  = (init_y + quad_dd);
		x2  = (init_x + quad_dd);
		y2  = (init_y + quad_dd * 2);
		x3  = (init_x + quad_dd * 2);
		y3  = (init_y + quad_dd * 3);
		love.graphics.line(x_c, y_c, x1, y1);
		love.graphics.line(x_c, y_c, x2, y2);
		love.graphics.line(x_c, y_c, x3, y3);

		-- down:
		init_x = dir_bars[3].x;
		init_y = dir_bars[3].y;
		x_c = (init_x + quad_dd * 2);
		y_c = (init_y + quad_dd * 3);
		x1  = (init_x + quad_dd);
		y1  = (init_y + quad_dd * 2);
		x2  = (init_x + quad_dd * 2);
		y2  = (init_y + quad_dd);
		x3  = (init_x + quad_dd * 3);
		y3  = (init_y + quad_dd * 2);
		love.graphics.line(x_c, y_c, x1, y1);
		love.graphics.line(x_c, y_c, x2, y2);
		love.graphics.line(x_c, y_c, x3, y3);

		-- up:
		local use_arrow = false;
		init_x = dir_bars[1].x;
		init_y = dir_bars[1].y;
		if use_arrow then
			x_c = (init_x + quad_dd * 2);
			y_c = (init_y + quad_dd);
			x1  = (init_x + quad_dd);
			y1  = (init_y + quad_dd * 2);
			x2  = (init_x + quad_dd * 2);
			y2  = (init_y + quad_dd * 3);
			x3  = (init_x + quad_dd * 3);
			y3  = (init_y + quad_dd * 2);
			love.graphics.line(x_c, y_c, x1, y1);
			love.graphics.line(x_c, y_c, x2, y2);
			love.graphics.line(x_c, y_c, x3, y3);
		else
			x_c = (init_x + quad_dd * 2);
			y_c = (init_y + quad_dd * 2);
			love.graphics.arc("line", x_c, y_c, quad_dd, 0, math.pi);

--[==[
			love.graphics.setColor(tetris_line_color, tetris_line_color, tetris_line_color);
			x1  = (init_x + quad_dd);
			y1  = (init_y + quad_dd * 2);
			local x1_1 = (init_x + quad_dd * 3);
			local y1_1 = (init_y + quad_dd * 2);
			love.graphics.line(x1, y1, x1_1, y1_1);
			love.graphics.setColor(tetris_font_color, tetris_font_color, tetris_font_color);
			x2  = (init_x + quad_dd * 2);
			y2  = (init_y + quad_dd * 2);
			x3  = (init_x + quad_dd);
			y3  = (init_y + quad_dd * 3);
			love.graphics.line(x1, y1, x2, y2);
			love.graphics.line(x1, y1, x3, y3);
--]==]

		end
		
	end

end

-- draw_replay:
function draw_replay()

	if tetris_state == enum_tetris_state_die then
		love.graphics.setColor(tetris_line_color, tetris_line_color, tetris_line_color)
		love.graphics.rectangle("fill", replay_bar_pos.x, replay_bar_pos.y, replay_bar_size.w, replay_bar_size.h)
		love.graphics.setColor(255,0,0)
		love.graphics.print("Replay", replay_bar_pos.x + 10, replay_bar_pos.y + 10)
	end

end

-- draw_pause:
function draw_pause()

	if tetris_state == enum_tetris_state_live then

		love.graphics.setColor(tetris_line_color, tetris_line_color, tetris_line_color)
		love.graphics.rectangle("line", pause_bar_pos.x, pause_bar_pos.y, pause_bar_size.w, pause_bar_size.h)
		local dx = pause_bar_size.w / 5
		local x = pause_bar_pos.x + dx
		love.graphics.rectangle("fill", x, pause_bar_pos.y, dx, pause_bar_size.h)
		x = x + dx + dx
		love.graphics.rectangle("fill", x, pause_bar_pos.y, dx, pause_bar_size.h)

	elseif tetris_state == enum_tetris_state_pause then

		love.graphics.setColor(tetris_line_color, tetris_line_color, tetris_line_color)
		love.graphics.rectangle("line", pause_bar_pos.x, pause_bar_pos.y, pause_bar_size.w, pause_bar_size.h)
		love.graphics.polygon('fill', pause_bar_pos.x, pause_bar_pos.y,
				(pause_bar_pos.x + pause_bar_size.w), (pause_bar_pos.y + pause_bar_size.h / 2),
				pause_bar_pos.x, (pause_bar_pos.y + pause_bar_size.h))

		-- confirm qiut dialog show:
		if game_pause_confirm_quit then
			draw_dlg_confirm_quit()
		end

	end

end

-- draw_single_block:
function draw_single_block(x, y, r, g, b)

	love.graphics.setColor(r, g, b)
	love.graphics.rectangle("fill", x, y, block_size, block_size)
	-- love.graphics.setColor(128,128,128)
	-- love.graphics.rectangle("line", x, y, block_size, block_size)

end

-- draw_next_block:
function draw_next_block()

	local rgb = tetris_block_tbl[tetris_block_next_type].color
	local r = tetris_color_tbl[rgb][1]
	local g = tetris_color_tbl[rgb][2]
	local b = tetris_color_tbl[rgb][3]

	draw_single_block(tetris_block_next_x, tetris_block_next_y, r, g, b)
	for i=1, tetris_block_tbl[tetris_block_next_type].block - 1 do
		local x = tetris_block_next_x + block_size * tetris_block_tbl[tetris_block_next_type].coordination[tetris_block_next_type_index][i].x
		local y = tetris_block_next_y + block_size * tetris_block_tbl[tetris_block_next_type].coordination[tetris_block_next_type_index][i].y
		draw_single_block(x, y, r, g, b)
	end

end

-- draw_cur_block:
function draw_cur_block()

	local rgb = tetris_block_tbl[tetris_block_cur_type].color
	local r = tetris_color_tbl[rgb][1]
	local g = tetris_color_tbl[rgb][2]
	local b = tetris_color_tbl[rgb][3]

	local ix = frame_beg_x + tetris_block_cur_ix * block_size
	local iy = frame_beg_y + tetris_block_cur_iy * block_size
	draw_single_block(ix, iy, r, g, b)
	for i=1, tetris_block_tbl[tetris_block_cur_type].block - 1 do
		local x = ix + block_size * tetris_block_tbl[tetris_block_cur_type].coordination[tetris_block_cur_type_index][i].x
		local y = iy + block_size * tetris_block_tbl[tetris_block_cur_type].coordination[tetris_block_cur_type_index][i].y
		draw_single_block(x, y, r, g, b)
	end

end

-- draw_frame_block:
function draw_frame_block()

	for i=0, block_y_count-1 do
		for j=0, block_x_count-1 do
			local idx = tetris_block_frame_tbl[i][j]
			if idx > 0 then
				local rgb = tetris_block_tbl[idx].color
				local r = tetris_color_tbl[rgb][1]
				local g = tetris_color_tbl[rgb][2]
				local b = tetris_color_tbl[rgb][3]
				local ix = frame_beg_x + j * block_size
				local iy = frame_beg_y + i * block_size
				draw_single_block(ix, iy, r, g, b)
			end
		end
	end

end

-- exchange_next_block:
function exchange_next_block()

	tetris_block_cur_type, tetris_block_cur_type_index = tetris_block_next_type, tetris_block_next_type_index
	tetris_block_cur_ix, tetris_block_cur_iy = adjust_init_pos(tetris_block_cur_type, tetris_block_cur_type_index)
	tetris_block_cur_ix = tetris_block_cur_ix + block_x_count / 2 - 2

	-- check died:
	is_died = false
	if tetris_block_frame_tbl[tetris_block_cur_iy][tetris_block_cur_ix] > 0 then
		is_died = true
	else
		for i=1, tetris_block_tbl[tetris_block_cur_type].block - 1 do
			local ix = tetris_block_cur_ix + tetris_block_tbl[tetris_block_cur_type].coordination[tetris_block_cur_type_index][i].x
			local iy = tetris_block_cur_iy + tetris_block_tbl[tetris_block_cur_type].coordination[tetris_block_cur_type_index][i].y
			if tetris_block_frame_tbl[iy][ix] > 0 then
				is_died = true
				break
			end
		end
	end

	if is_died then
		-- game over:
		tetris_state  = enum_tetris_state_die
		game_end_time = os.time()
		write_json(true)
	else
		-- aux:
		if tetris_aux then
			tetris_aux_func_calc()
		end
	end

end

-- gen_rand_block:
function gen_rand_block()

	local t = math.random(7)
	local i = math.random(4)
	return t, i

end

-- gen_next_block:
function gen_next_block()

	tetris_block_next_type, tetris_block_next_type_index = gen_rand_block()
	tetris_block_next_x, tetris_block_next_y = adjust_init_pos(tetris_block_next_type, tetris_block_next_type_index)
	tetris_block_next_x = frame_next_beg_x + tetris_block_next_x * block_size
	tetris_block_next_y = frame_next_beg_y + tetris_block_next_y * block_size

end

-- check_block_pos:
function check_block_pos(t, i, ix, iy)

	local ret = true

	if ix < 0 or iy < 0 or ix >= block_x_count or iy >= block_y_count or tetris_block_frame_tbl[iy][ix] > 0 then
		ret = false
	else
		for j=1, tetris_block_tbl[t].block - 1 do
			local tmp_ix = ix + tetris_block_tbl[t].coordination[i][j].x
			if tmp_ix < 0 or tmp_ix >= block_x_count then
				ret = false
				break
			end
			local tmp_iy = iy + tetris_block_tbl[t].coordination[i][j].y
			if tmp_iy < 0 or tmp_iy >= block_y_count then
				ret = false
				break
			end
			if tetris_block_frame_tbl[tmp_iy][tmp_ix] > 0 then
				ret = false
				break
			end
		end
	end

	return ret

end


-- check_rotate_cur_block:
function check_rotate_cur_block()

	local tmp_type_index = tetris_block_cur_type_index + 1
	if tmp_type_index > 4 then
		tmp_type_index = 1
	end

	return check_block_pos(tetris_block_cur_type, tmp_type_index, tetris_block_cur_ix, tetris_block_cur_iy)

end

-- rotate_cur_block:
function rotate_cur_block()

	tetris_block_cur_type_index = tetris_block_cur_type_index + 1
	if tetris_block_cur_type_index > 4 then
		tetris_block_cur_type_index = 1
	end

	-- aux:
	if tetris_aux then
		tetris_aux_func_calc()
	end

end

-- check_move_cur_block:
function check_move_cur_block(dx, dy)

	local ret = true
	local tmp_ix = tetris_block_cur_ix + dx
	local tmp_iy = tetris_block_cur_iy + dy

	return check_block_pos(tetris_block_cur_type, tetris_block_cur_type_index, tmp_ix, tmp_iy)

end

-- move_cur_block:
function move_cur_block(dx, dy)

	tetris_block_cur_ix = tetris_block_cur_ix + dx
	tetris_block_cur_iy = tetris_block_cur_iy + dy

	-- aux:
	if tetris_aux then
		tetris_aux_func_calc()
	end

end

-- draw_cur_info:
function draw_cur_info()

	love.graphics.setColor(tetris_line_color, tetris_line_color, tetris_line_color)
	love.graphics.rectangle("line", info_rect.x, info_rect.y, info_rect.w, info_rect.h)

	love.graphics.setColor(tetris_font_color, tetris_font_color, tetris_font_color)
	local str_line     = string.format("L: %d", tetris_line)
	local str_score    = string.format("S: %d", tetris_score)
	local str_line_hi  = string.format("L: %d", json_tbl.line)
	local str_score_hi = string.format("S: %d", json_tbl.score)

	if tetris_state == enum_tetris_state_live then
		local duration = (os.time() - game_start_time - game_pause_duration)
		game_duration_str = fmt_duration(duration)
	end

	love.graphics.setFont(tetris_font);

	local y_indent = 20
	local ix = info_rect.x + 4
	local iy = info_rect.y + 4
	love.graphics.print("SCORE:",  ix, iy)
	iy = iy + y_indent
	love.graphics.print(str_line,  ix, iy)
	iy = iy + y_indent
	love.graphics.print(str_score, ix, iy)
	iy = iy + y_indent
	love.graphics.print(game_duration_str, ix, iy)
	iy = (iy + y_indent + 8)
	love.graphics.print("HI-SCORE:",  ix, iy)
	iy = iy + y_indent
	love.graphics.print(str_line_hi,  ix, iy)
	iy = iy + y_indent
	love.graphics.print(str_score_hi, ix, iy)
	iy = iy + y_indent
	love.graphics.print(json_tbl.timeusedstr, ix, iy)

end

-- check_die:
function check_die()

	local is_die = false
	if tetris_block_cur_ix < 0 then
		is_die = true
	else
		for i=1, tetris_block_tbl[tetris_block_cur_type].block - 1 do
			local ix = tetris_block_cur_ix + tetris_block_tbl[tetris_block_cur_type].coordination[tetris_block_cur_type_index][i].x
			if ix < 0 then
				is_die = true
				break
			end
		end
	end

	return is_die

end

-- falldown_cur_block:
dtotal = 0
function falldown_cur_block()

	if check_move_cur_block(0, 1) then

		move_cur_block(0, 1)

	else

		-- add cur block to frame:
		for i=1, tetris_block_tbl[tetris_block_cur_type].block - 1 do
			local ix = tetris_block_cur_ix + tetris_block_tbl[tetris_block_cur_type].coordination[tetris_block_cur_type_index][i].x
			local iy = tetris_block_cur_iy + tetris_block_tbl[tetris_block_cur_type].coordination[tetris_block_cur_type_index][i].y
			tetris_block_frame_tbl[iy][ix] = tetris_block_cur_type
		end
		tetris_block_frame_tbl[tetris_block_cur_iy][tetris_block_cur_ix] = tetris_block_cur_type

		------ check full line:
		full_line_count = 0

		for i=0, block_y_count-1 do
			local c = 0
			for j=0, block_x_count-1 do
				if tetris_block_frame_tbl[i][j] > 0 then
					c = c + 1
				else
					break
				end
			end
			if c == block_x_count then
				full_line_count = full_line_count + 1
				full_line_tbl[full_line_count] = i
			end
		end

		if full_line_count > 0 then

			-- construct diminish:
			love.graphics.setCanvas(diminish_canvas);
				love.graphics.clear();
				love.graphics.setColor(64, 64, 64);
				for k = 1, full_line_count do
					love.graphics.rectangle('fill', 0, full_line_tbl[k] * block_size, frame_width, block_size);
				end
			love.graphics.setCanvas();
			diminish_tick = 5;

		else

			----------------------------
			exchange_next_block();
			gen_next_block();
			----------------------------

		end

	end

end

-----------------------------------------------------------------------------
-- tetris_aux_calc:
function tetris_aux_func_calc()

	tetris_aux_line_beg_x = tetris_block_cur_ix
	tetris_aux_line_end_x = tetris_block_cur_ix
	for i=1, tetris_block_tbl[tetris_block_cur_type].block - 1 do
		local ix = tetris_block_cur_ix + tetris_block_tbl[tetris_block_cur_type].coordination[tetris_block_cur_type_index][i].x
		if ix < tetris_aux_line_beg_x then
			tetris_aux_line_beg_x = ix
		end
		if ix > tetris_aux_line_end_x then
			tetris_aux_line_end_x = ix
		end
	end
	tetris_aux_line_end_x = tetris_aux_line_end_x + 1

end

-- tetris_aux_func_draw:
function tetris_aux_func_draw()

	love.graphics.setColor(tetris_font_color, tetris_font_color, tetris_font_color)
	local x = frame_beg_x + tetris_aux_line_beg_x * block_size
	love.graphics.line(x, frame_beg_y, x, frame_end_y)
	x = frame_beg_x + tetris_aux_line_end_x * block_size
	love.graphics.line(x, frame_beg_y, x, frame_end_y)

end
-----------------------------------------------------------------------------

-- debug_print:
function debug_print()

	local ix = 10
	local iy = 10
	local y_indent = 22
	love.graphics.setColor(tetris_font_color, tetris_font_color, tetris_font_color)
	local str = string.format("FPS_: %d", love.timer.getFPS())
	love.graphics.print(str, ix, iy)
	str = string.format("NEXT: %d, %d", tetris_block_next_type, tetris_block_next_type_index)
	iy = iy + y_indent
	love.graphics.print(str, ix, iy)
	str = string.format("CURR: %d, %d", tetris_block_cur_type, tetris_block_cur_type_index)
	iy = iy + y_indent
	love.graphics.print(str, ix, iy)
	str = string.format("CPOS: %d, %d", tetris_block_cur_ix, tetris_block_cur_iy)
	iy = iy + y_indent
	love.graphics.print(str, ix, iy)

end

-- draw_sysinfo:
function draw_sysinfo()

	love.graphics.setColor(tetris_font_color, tetris_font_color, tetris_font_color)
	local str = string.format("FPS: %d", love.timer.getFPS())
	love.graphics.print(str, (scr_width - 120), 10)

end

-- love.load:
function love.load()

	math.randomseed(os.time());

	read_json();

	init_tetris_block_tbl_init();
	init_tetris_block_frame_tbl();

	reload_tetris();

end

-- love.draw:
function love.draw()

	draw_next_block()
	draw_frame_block()
	draw_cur_block()

	if diminish_tick > 0 then
		if diminish_tick % 2 == 1 then
			love.graphics.draw(diminish_canvas, frame_beg_x, frame_beg_y)
		end
	end

	draw_frame()

	draw_dirbar()
	draw_replay()
	draw_pause()

	-- aux:
	if tetris_aux then
		tetris_aux_func_draw()
	end

	draw_cur_info()
	show_soft_info(scr_width, scr_height)
	draw_sysinfo()
	-- debug_print()

end

-- deal_keypressed:
function deal_keypressed(key)

	if (key == "up" or key == "w" ) then

		if check_rotate_cur_block() then
			rotate_cur_block()
		end

	elseif (key == "down" or key == "s") then

		falldown_cur_block()

	elseif (key == "left" or key == "a") then

		if check_move_cur_block(-1, 0) then
			move_cur_block(-1, 0)
		end

	elseif (key == "right" or key == "d") then

		if check_move_cur_block(1, 0) then
			move_cur_block(1, 0)
		end
	end

end

-- love.keypressed:
dtotal_key = 0
function love.keypressed(key)

	if (key == "escape") then

		if tetris_state == enum_tetris_state_live then
			tetris_state = enum_tetris_state_pause
			game_pause_confirm_quit = true
			game_pause_start_time = os.time()
		elseif tetris_state == enum_tetris_state_pause then
			if game_pause_confirm_quit then
				love.event.quit()
			else
				game_pause_confirm_quit = true
			end
		else
			love.event.quit()
		end
	end

	if tetris_state == enum_tetris_state_live then

		if (key == "up" or key == "w" ) then

			dtotal_key = 0
			if check_rotate_cur_block() then
				rotate_cur_block()
			end

		elseif (key == "down" or key == "s") then

			dtotal_key = 0
			falldown_cur_block()

		elseif (key == "left" or key == "a") then

			dtotal_key = 0
			if check_move_cur_block(-1, 0) then
				move_cur_block(-1, 0)
			end

		elseif (key == "right" or key == "d") then

			dtotal_key = 0
			if check_move_cur_block(1, 0) then
				move_cur_block(1, 0)
			end
		end

	else

		if (key == "return") then
			reload_tetris()
		end

	end

end

-- love.mousepressed:
function love.mousepressed(x, y, button, istouch)

	local processed = false
	if tetris_state == enum_tetris_state_pause then
		if game_pause_confirm_quit then
			local nRet = dlg_confirm_quit_click(x, y)
			if nRet == enum_dlg_confirm_btn_ok then
				love.event.quit()
			elseif nRet == enum_dlg_confirm_btn_cancel then
				game_pause_confirm_quit = false
			end
			processed = true
		end
	end

	for i = 1, #dir_bars do
		if x >= dir_bars[i].x and x <= (dir_bars[i].x + dir_bar_size)
				and y >= dir_bars[i].y and  y <= (dir_bars[i].y + dir_bar_size) then
			love.keypressed(dir_bars[i].dir)
			dtotal_key = 0
			processed = true
			break
		end
	end

	if not processed then
		if tetris_state == enum_tetris_state_die then
			if x >= replay_bar_pos.x and x <= (replay_bar_pos.x + replay_bar_size.w)
					and y >= replay_bar_pos.y and y <= (replay_bar_pos.y + replay_bar_size.h) then
				reload_tetris()
				processed = true
			end
		end
	end

	if not processed then
		if tetris_state == enum_tetris_state_live or tetris_state == enum_tetris_state_pause then
			if x >= pause_bar_pos.x and x <= (pause_bar_pos.x + pause_bar_size.w)
					and y >= pause_bar_pos.y and y <= (pause_bar_pos.y + pause_bar_size.h) then
				if tetris_state == enum_tetris_state_live then
					tetris_state = enum_tetris_state_pause
					game_pause_start_time = os.time()
				else
					tetris_state = enum_tetris_state_live
					game_pause_duration = (game_pause_duration + (os.time() - game_pause_start_time))
				end
				processed = true
			end
		end
	end

end

-- love.update:
function love.update(dt)

	if tetris_state == enum_tetris_state_live then

		dtotal_key = dtotal_key + dt

		if dtotal_key > 0.12 then

			if diminish_tick > 0 then

				diminish_tick = diminish_tick - 1
				if diminish_tick == 0 then

					-- earse full line:
					for i=1, full_line_count do
						local j = full_line_tbl[i]
						while j > 0 do
							for k=0, block_x_count-1 do
								tetris_block_frame_tbl[j][k] = tetris_block_frame_tbl[j-1][k]
							end
							j = j - 1
						end
						for k=0, block_x_count-1 do
							tetris_block_frame_tbl[0][k] = 0
						end
					end
					----------------------------
					tetris_line  = tetris_line + full_line_count
					tetris_score = tetris_score + (full_line_count * full_line_count - (full_line_count - 1)) * 100
					----------------------------
					exchange_next_block();
					gen_next_block();
					----------------------------

					dtotal = 0;

				end

			else

				if love.mouse.isDown(1) then

					local x, y = love.mouse.getPosition()
					for i = 1, #dir_bars do
						if x >= dir_bars[i].x and x <= (dir_bars[i].x + dir_bar_size)
								and y >= dir_bars[i].y and  y <= (dir_bars[i].y + dir_bar_size) then
							deal_keypressed(dir_bars[i].dir)
							break
						end
					end

				else

					if love.keyboard.isDown("up") or love.keyboard.isDown("w") then
						deal_keypressed("up")
					elseif love.keyboard.isDown("down") or love.keyboard.isDown("s") then
						deal_keypressed("down")
					elseif love.keyboard.isDown("left") or love.keyboard.isDown("a") then
						deal_keypressed("left")
					elseif love.keyboard.isDown("right") or love.keyboard.isDown("d") then
						deal_keypressed("right")
					end

				end

			end

			dtotal_key = 0

		end

		if diminish_tick <= 0 then

			dtotal = dtotal + dt
			if dtotal >= 0.65 then
				falldown_cur_block()
				dtotal = 0
			end

		end

	end

end

